import React from "react";
const styles = {};
const rootCommon = {
  display: "inline-block",
  boxSizing: "border-box",
  maxWidth: "100%",
  minWidth: "100px",
  overflow: "hidden",
  verticalAlign: "bottom",
  padding: 2
};
const rootStyled = {
  border: "1px solid #ccc",
  backgroundColor: "white"
};
styles.root = {
  ...rootCommon,
  overflow: "hidden"
};
styles.rootStyled = {
  ...styles.root,
  ...rootStyled
};
styles.rootMl = {
  ...rootCommon,
  overflow: "auto"
};
styles.rootMlStyled = {
  ...styles.rootMl,
  ...rootStyled
};
styles.textareaWrap = {
  display: "block",
  position: "relative",
  width: "100%"
};
styles.unscroller = {
  display: "inline-block",
  overflowX: "scroll",
  overflowY: "hidden",
  position: "relative",
  width: "100%"
};
const inputCommon = {
  fontSize: "14px",
  fontFamily: "sans-serif",
  boxSizing: "border-box",
  borderWidth: "0",
  minWidth: "100%",
  margin: "0",
  padding: "0",
  lineHeight: "20px",
  whiteSpace: "pre"
};
styles.input = {
  ...inputCommon,
  position: "relative",
  zIndex: 1,
  verticalAlign: "text-bottom",
  outline: "none",
  caretColor: "black",
  backgroundColor: "transparent",
  color: "rgba(0,0,0,0)",
  resize: "none",
  overflow: "hidden"
};
styles.stub = {
  display: "block",
  position: "fixed",
  left: "-100000px",
  opacity: 0,
  whiteSpace: "pre"
};
styles.placeholder = {
  ...inputCommon,
  position: "absolute",
  zIndex: 0,
  top: 0,
  left: 0,
  display: "block",
  overflow: "hidden",
  whiteSpace: "pre"
};
const scrollbar = getScrollbarSize();
class InputData {
  constructor(inputType, data, selectionStart, selectionEnd) {
    this.inputType = inputType;
    this.data = data;
    this.selectionStart = selectionStart;
    this.selectionEnd = selectionEnd === void 0 ? selectionStart : selectionEnd;
  }
}
export default class MightyInput extends React.Component {
  static defaultProps = {
    filter: (v) => v,
    render: (v) => v,
    onChange: () => {
    },
    onUpdate: () => {
    },
    isMultiline: false,
    rows: 1,
    offsetRight: 0
  };
  state = {
    value: this.props.value ?? "",
    input: new InputData()
  };
  #rootRef = React.createRef();
  #unscrollerRef = React.createRef();
  #placeholderRef = React.createRef();
  #inputRef = React.createRef();
  #stubRef = React.createRef();
  get rootEl() {
    return this.#rootRef.current;
  }
  get unscrollerEl() {
    return this.#unscrollerRef.current;
  }
  get inputEl() {
    return this.#inputRef.current;
  }
  get placeholderEl() {
    return this.#placeholderRef.current;
  }
  get stubEl() {
    return this.#stubRef.current;
  }
  componentDidMount() {
    this.inputEl.addEventListener("beforeinput", (e) => {
      this.setState({
        input: new InputData(e.inputType, e.data, e.target.selectionStart, e.target.selectionEnd)
      });
    });
    this.#initializeDom();
    this.#updateSize(this.state.value);
  }
  #onChange = (e) => {
    this.props.onChange?.(e);
  };
  #onInput = (e) => {
    this.#handleInput(e);
    this.props.onInput?.(e);
  };
  #handleInput(e) {
    const { value, input } = this.state;
    const { filter } = this.props;
    const newValue = filter(e.target.value, value, input);
    if (value !== newValue) {
      this.setState({
        value: newValue,
        input: new InputData()
      });
      this.#updateSize(newValue);
      this.props.onUpdate(newValue, value, input);
    }
  }
  #initializeDom() {
    const { stubEl, rootEl, inputEl, unscrollerEl, props } = this;
    const inputStyle = window.getComputedStyle(inputEl);
    const rootStyle = window.getComputedStyle(rootEl);
    stubEl.style.padding = inputStyle.padding;
    stubEl.style.border = inputStyle.border;
    stubEl.style.fontFamily = inputStyle.fontFamily;
    stubEl.style.fontSize = inputStyle.fontSize;
    stubEl.style.lineHeight = inputStyle.lineHeight;
    stubEl.textContent = "\xA0\n".repeat(props.rows);
    rootEl.style.height = parseInt(rootStyle.paddingTop, 10) + parseInt(rootStyle.paddingBottom, 10) + parseInt(rootStyle.borderTopWidth || 0, 10) + parseInt(rootStyle.borderBottomWidth || 0, 10) + stubEl.offsetHeight + "px";
    unscrollerEl.style.height = rootEl.offsetHeight + (!props.isMultiline ? scrollbar.height || 20 : 0) + "px";
  }
  #updateSize(value) {
    const {
      stubEl,
      placeholderEl,
      inputEl,
      unscrollerEl,
      props
    } = this;
    let linesCount = countLines(value, props.rows);
    let tail = "\n".repeat(Math.max(0, props.rows - linesCount));
    stubEl.textContent = value + tail + "\xA0";
    const { offsetWidth, offsetHeight } = stubEl;
    placeholderEl.style.width = inputEl.style.width = offsetWidth + props.offsetRight + "px";
    placeholderEl.style.height = inputEl.style.height = offsetHeight + "px";
    if (props.isMultiline) {
      unscrollerEl.style.height = offsetHeight + "px";
    }
  }
  render() {
    const {
      render,
      filter,
      className,
      onUpdate,
      style,
      modifiers,
      isMultiline,
      rows,
      offsetRight,
      ...props
    } = this.props;
    const { value } = this.state;
    let rootStyle;
    if (className) {
      rootStyle = isMultiline ? styles.rootMl : styles.root;
    } else {
      rootStyle = isMultiline ? styles.rootMlStyled : styles.rootStyled;
    }
    let renderedValue = render(value) ?? "";
    if (!renderedValue.length) {
      renderedValue = "\xA0";
    }
    let InputComponent = "input";
    if (isMultiline) {
      InputComponent = "textarea";
    }
    return /* @__PURE__ */ React.createElement("div", {
      ref: this.#rootRef,
      className: className ?? "",
      style: rootStyle
    }, /* @__PURE__ */ React.createElement("div", {
      ref: this.#unscrollerRef,
      style: isMultiline ? styles.textareaWrap : styles.unscroller
    }, /* @__PURE__ */ React.createElement(InputComponent, {
      ...props,
      style: styles.input,
      ref: this.#inputRef,
      onChange: this.#onChange,
      onInput: this.#onInput,
      value
    }), /* @__PURE__ */ React.createElement("div", {
      "aria-hidden": true,
      style: styles.placeholder,
      ref: this.#placeholderRef
    }, renderedValue), /* @__PURE__ */ React.createElement("div", {
      "aria-hidden": true,
      ref: this.#stubRef,
      style: styles.stub
    })));
  }
}
function getScrollbarSize() {
  const root = document.createElement("div");
  root.style.position = "absolute";
  root.style.top = "-100px";
  const outer = document.createElement("div");
  outer.style.position = "relative";
  outer.style.width = "100px";
  outer.style.height = "100px";
  outer.style.overflow = "scroll";
  const inner = document.createElement("div");
  inner.style.position = "absolute";
  inner.style.top = "0px";
  inner.style.bottom = "0px";
  inner.style.right = "0px";
  inner.style.left = "0px";
  outer.appendChild(inner);
  root.appendChild(outer);
  document.body.appendChild(root);
  const width = outer.offsetWidth - inner.offsetWidth;
  const height = outer.offsetHeight - inner.offsetHeight;
  document.body.removeChild(root);
  return { width, height };
}
function countLines(string, threshold = Infinity) {
  let count = 1;
  let lastIndex = -1;
  while (true) {
    if (count >= threshold) {
      return count;
    }
    let i = string.indexOf("\n", lastIndex);
    if (i > -1) {
      count += 1;
      lastIndex = i + 1;
    } else {
      break;
    }
  }
  return count;
}
//# sourceMappingURL=mighty-input.js.map
